# Pipeline configuration for gstreamer

# We no longer use these overlays (although they are still supported
# by the application). Instead, we write a matching .srt subtitle
# file with the data - this improves encoding/playback performance.
# ! timeoverlay \
# ! textoverlay text="$PERSONAL_DATA" valign=bottom halign=center \
#               font-desc="Bitstream Vera Sans"

# Video/audio encoding with XV output and all-in-one muxing
# of audio into video
PIPELINE_MUX="""v4l2src device=/dev/video0 name=source use-fixed-fps=false \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! tee name=screen \
	! theoraenc quality=56 \
	! queue \
	! oggmux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ screen. \
	! xvimagesink } \ 
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! vorbisenc \
	! queue \
	! mux. }
	"""

# Test video source and audio encoding for audio only recording (so subtitles can 
# still be displayed).
PIPELINE_MUX_AUDIOONLY="""videotestsrc name=source pattern=black \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! theoraenc \
	! queue \
	! oggmux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! vorbisenc \
	! queue \
	! mux. }
	"""

# Video/audio encoding (xvid/mp3) with XV output and all-in-one muxing
# of audio into video
PIPELINE_XVID_MUX="""v4l2src device=/dev/video0 name=source use-fixed-fps=false \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! tee name=screen \
	! xvidenc \
	! queue \
	! avimux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ screen. \
	! xvimagesink } \ 
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! lame \
	! queue \
	! mux. }
	"""

# Test (no cam) video/audio encoding with compositing, XV output and
# muxing of audio into video
PIPELINE_TEST_MUX="""videotestsrc name=source \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! tee name=screen \
	! theoraenc quality=56 \
	! queue \
	! oggmux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ screen. \
	! xvimagesink } \
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! vorbisenc \
	! queue \
	! mux. }
	"""

# Test (no cam) video/audio encoding with compositing, XV output and
# muxing of audio into video
PIPELINE_TEST_MUX_AUDIOONLY="""videotestsrc name=source pattern=black \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! theoraenc quality=56 \
	! queue \
	! oggmux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! vorbisenc \
	! queue \
	! mux. }
	"""

# Video/audio encoding (xvid/mp3) with XV output and all-in-one muxing
# of audio into video
PIPELINE_TEST_XVID_MUX="""videotestsrc name=source \
	! videorate \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=320,height=240,framerate=10/1 \
	! ffmpegcolorspace \
	! tee name=screen \
	! xvidenc \
	! queue \
	! avimux name=mux \
	! filesink location=$OUTPUT_FILE \
	{ screen. \
	! xvimagesink } \ 
	{ alsasrc \
	! audiorate \
        ! audio/x-raw-int,rate=11025,depth=16,channels=1,width=16,sign=(boolean)TRUE,endianness=1234 \
	! audioconvert \
	! lame \
	! queue \
	! mux. }
	"""

# Select pipeline:
# PIPELINE_MUX: All-in-one OGG encoding
# PIPELINE_MUX_AUDIOONLY: All-in-one OGG encoding for audio only (no xv sink)
# PIPELINE_XVID_MUX: All-in-one XVID/MP3 encoding
# PIPELINE_TEST_MUX: Uses test video stream (no cam)
# PIPELINE_TEST_MUX_AUDIOONLY: Same as mux audioonly
# PIPELINE_TEST_XVID_MUX: Uses test video stream (no cam)
PIPELINE=PIPELINE_TEST_MUX
PIPELINE_AUDIO=PIPELINE_TEST_MUX_AUDIOONLY
