import gc
import sys
import pygtk
pygtk.require('2.0')
import gobject
import time
import pygst
pygst.require('0.10')
import gst
from config import *
from pipeline import *

"""

	Video Booth Kiosk Application v2 recording component
	Author: Robin Rawson-Tetley

"""


class MainWindow(gtk.Window):

	def destroy(self, widget=None, data=None):
		gtk.main_quit()

	def delete_event(self, widget, event, data=None):
		return False

	def loadfromfile(self, filename):
		"""
		Loads the contents of a file and returns it as a string
		"""
		f = open(filename)
		out = ""
		while True:
			s = f.readline()
			if s == "": break
			out = out + s
		del f
		return out

	def recorddiary(self, widget=None, data=None):
		# Kick off the flashing recording light
		self.reclight = True
		self.showreclight = True
		gobject.timeout_add(500, self.flashreclight)
		# Set stop button as default
		self.btnstoprecord.grab_focus()
		# Start recording - we do it in idle so that GTK can
		# display the drawable before gstreamer starts painting
		# onto it.
		gobject.idle_add(self.startrecording)

	def startrecording(self, widget=None, data=None):
		# Mark us as recording and the start time
		self.isrecording = True
		self.starttime = time.time()
		if self.isaudioonly:
			pipelinestr = PIPELINE_AUDIO
		else:
			pipelinestr = PIPELINE
		# Output file name
		pipelinestr = pipelinestr.replace("$OUTPUT_FILE", self.moviefile)
		# Create the gstreamer pipeline
		try:
			self.pipeline = gst.parse_launch(pipelinestr)
		except gobject.GError, e:
			self.error('Could not create pipeline', str(e))
			self.isrecording = False
			return

		# If we're doing video, display the xv drawable and 
		# find the xvimagesink and tell it to use our image
		if not self.isaudioonly:
			self.imaudioonly.hide()
			self.da.show()
			elements = self.pipeline.elements()
			for i in elements:
				if i.get_name().startswith("xvimagesink"):
					i.set_xwindow_id(self.da.window.xid)
					break
			del elements
		else:
			# Otherwise, replace the drawable with an audio only
			# image
			self.da.hide()
			self.imaudioonly.show()

		# Start the pipeline going
		self.pipeline.set_state(gst.STATE_PLAYING)

		# Clean up
		del pipelinestr

		# Start our timeout check going to see if we should
		# stop if the recording time exceeds our maximum length
		gobject.timeout_add(1000, self.checktimeout)

		# Called in idle - make sure it isn't called again
		return False


	def checktimeout(self, widget=None, data=None):
		# Has the user stopped recording? Stop the timer if they have
		if not self.isrecording: return False
		# Have we gone past our recording time?
		if time.time() > (self.starttime + MAX_MOVIETIME_S):
			# Stop this timer and stop the recording
			self.stoprecording()
			return False
		else:
			return True
	
	def stoprecording(self, widget=None, data=None):
		# If the user already stopped it, don't do anything here -
		# we've been called by the movie timeout
		if not self.isrecording: return 
		# Mark us as stopped
		self.isrecording = False
		# Stop the recording light
		self.showreclight = False
		# Stop gstreamer pipeline
		try:
			# Stop the gstreamer pipeline
		        self.pipeline.set_state(gst.STATE_NULL)
		except:
			pass
		# Free reference to the pipeline
		del self.pipeline
		# Garbage collection debugging
		# gc.set_debug(gc.DEBUG_LEAK)
		# print "gc: %s uncollectable objects" % gc.collect()
		# print gc.garbage
		# Force garbage collection
		gc.collect()
		# Show thank you page
		self.layout.remove(self.ctrecord)
		self.layout.add(self.ctthanks)
		self.layout.show_all()
		# Set timer to close the recording component
		gobject.timeout_add(2500, self.destroy)

	def error(self, message, secondary=None):
		"""
		Output error dialog for gstreamer errors
		"""
		m = gtk.MessageDialog(self,
			      gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
			      gtk.MESSAGE_ERROR,
			      gtk.BUTTONS_OK,
			      message)
		if secondary:
			m.format_secondary_text(secondary)
			m.run()
			m.destroy()
			self.stoprecording()

	def flashreclight(self, widget=None, data=None):
		if self.reclight:
			self.reclight = False
			self.imrecord.hide()
		else:
			self.reclight = True
			self.imrecord.show()
		return self.showreclight

	def imagebutton(self, img_filename, label_text):
		""" Creates an icon/label for use on buttons """
		box1 = gtk.HBox(False, 0)
	        box1.set_border_width(2)
		image = gtk.Image()
		image.set_from_file(img_filename)
		label = gtk.Label(label_text)
		box1.pack_start(image, False, False, 3)
		box1.pack_start(label, False, False, 3)
		image.show()
		label.show()
		return box1

	def __init__(self, audioonly, moviefile):
		
		# Default values =======================
		self.isaudioonly = audioonly
		self.moviefile = moviefile
		self.isrecording = False

		# Window stuff =========================
		gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
		self.connect("delete_event", self.delete_event)
		self.connect("destroy", self.destroy)
		self.set_title("Video Booth Record")
		# Disable decoration
		self.set_decorated(False)
		self.layout = gtk.VBox(1)
		
		
		# Diary recording screen =============
		# Screen layout - Fixed co-ordinates
		self.ctrecord = gtk.Fixed()
		# Output an image for audio only - the drawable will
		# overlay it when video is active
		self.imaudioonly = gtk.Image()
		self.imaudioonly.set_from_file("images/audioonly.png")
		movieposx = (SCREEN_WIDTH - CAMRES_WIDTH) / 2
		movieposy = (SCREEN_HEIGHT - CAMRES_HEIGHT) / 2
		self.ctrecord.put(self.imaudioonly, movieposx, movieposy)
		# Output drawable for image sink
		self.da = gtk.DrawingArea()
		self.da.set_size_request(CAMRES_WIDTH, CAMRES_HEIGHT)
		self.ctrecord.put(self.da, movieposx, movieposy)
		# Recording light
		self.imrecord = gtk.Image()
		self.imrecord.set_from_file("icons/reclight_on.png")
		self.ctrecord.put(self.imrecord, movieposx - 50, movieposy)
		# Stop button
		self.btnstoprecord = gtk.Button()
		self.btnstoprecord.add(self.imagebutton("icons/stop.png", "Stop"))
		self.btnstoprecord.connect("clicked", self.stoprecording)
		self.ctrecord.put(self.btnstoprecord, movieposx + (CAMRES_WIDTH / 2) - 25, movieposy + CAMRES_HEIGHT + 20)


		# Thanks screen (post record) ======
		self.ctthanks = gtk.VBox()
		self.ctthanks.add(gtk.Label(self.loadfromfile("thanks.txt")))






		# Recording screen
		self.layout.add(self.ctrecord)
		# Display our window
		self.layout.show_all()
		self.add(self.layout)
		self.move(0, 0)
		self.resize(SCREEN_WIDTH, SCREEN_HEIGHT)
		self.realize()
		self.show()
		# Load background pixmap and cache it
		self.pixbuf = gtk.gdk.pixbuf_new_from_file("images/bg_%sx%s.jpg" % (SCREEN_WIDTH, SCREEN_HEIGHT))
		self.pixmap, self.mask = self.pixbuf.render_pixmap_and_mask()
		self.set_app_paintable(True)
		self.window.set_back_pixmap(self.pixmap, False)
		del self.pixbuf
		del self.mask
		del self.pixmap

		# Go straight into recording
		self.recorddiary()

	def main(self):
		gtk.threads_init()
		gtk.main()









if __name__ == "__main__":
	if len(sys.argv) == 3:
		audioonly = sys.argv[1] == "true"
		moviefile = sys.argv[2]
		m = MainWindow(audioonly, moviefile)
		m.main()
	else:
		print "Usage: record.py audioonly(true|false), moviefilename"
