#!/usr/bin/env python

import pygtk
pygtk.require('2.0')
import gtk
import gtk.gdk
import pango
import gobject
import pygst
pygst.require('0.10')
import gst

PIPELINE="""v4l2src device=/dev/video0 name=source use-fixed-fps=false \
	! video/x-raw-yuv,format=(fourcc)UYVY,width=640,height=480 \
	! ffmpegcolorspace \
	! tee name=screen \
	! timeoverlay \
	! textoverlay text="some text" valign=bottom halign=center \
	  font-desc="Bitstream Vera Sans"
	! theoraenc \
	! oggmux name=mux \
	! filesink location=video.ogg \
	screen. \
	! xvimagesink \ 
	alsasrc 
	! audioconvert
	! vorbisenc
	! queue
	! mux.
	"""

class Window(gtk.Window):
    def __init__(self):
        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.playing = False
        self.selected_pipe = None
        self.pipeline = None
        self.prepare_ui()

    def prepare_ui(self):
	self.set_title('Capture')
        self.connect('delete-event', lambda *x: gtk.main_quit())
	self.layout = gtk.VBox(1)

	self.c = gtk.VBox(2)
	self.c.show()
	self.add(self.c)

	self.da = gtk.DrawingArea()
	self.da.set_size_request(640,480)
	self.c.add(self.da)
	self.da.show()

	self.buttons = gtk.HButtonBox()
	self.buttons.show()

	self.c.add(self.buttons)

	self.go = gtk.Button("Go")
	self.go.connect('clicked', self.play)
	self.go.show()
	self.buttons.add(self.go)

	self.stop = gtk.Button("Stop")
	self.stop.connect('clicked', self.stoprecording)
	self.stop.show()
	self.buttons.add(self.stop)

    def on_message(self, bus, message):
        t = message.type
        print message
        if t == gst.MESSAGE_STATE_CHANGED:
            pass
        elif t == gst.MESSAGE_ERROR:
            err, debug = message.parse_error()
            self.error("%s" % err, debug)
        elif t == gst.MESSAGE_EOS:
            self.play_toggled()
        else:
            print '%s: %s:' % (message.src.get_path_string(),
                               message.type.value_nicks[1])
            if message.structure:
                print '    %s' % message.structure.to_string()
            else:
                print '    (no structure)'
        return True

    def error(self, message, secondary=None):
        m = gtk.MessageDialog(self,
                              gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                              gtk.MESSAGE_ERROR,
                              gtk.BUTTONS_OK,
                              message)
        if secondary:
            m.format_secondary_text(secondary)
        m.run()
        m.destroy()
        self.stop()

    def play(self,widget=None,data=None):
        try:
            self.set_sensitive(False)
            pipeline = gst.parse_launch(PIPELINE)
            self.set_sensitive(True)
        except gobject.GError, e:
            self.set_sensitive(True)
            self.error('Could not create pipeline', str(e))
            return False
        
        bus = pipeline.get_bus()
        bus.add_signal_watch()
        watch_id = bus.connect('message', self.on_message)
        self.pipeline = pipeline
        self.watch_id = watch_id
	
	# Find the xvimagesink and tell it to use our image
	self.sink = None
	for i in self.pipeline.elements():
		if i.get_name() == "xvimagesink0":
			self.sink = i
	self.sink.set_xwindow_id(self.da.window.xid)

        pipeline.set_state(gst.STATE_PLAYING)

    def stoprecording(self,widget=None,data=None):
        bus = self.pipeline.get_bus()
        bus.disconnect(self.watch_id)
        bus.remove_signal_watch()
        self.pipeline.set_state(gst.STATE_NULL)
        self.pipeline = None
        del self.watch_id

if __name__ == '__main__':
    w = Window()
    w.show()
    gtk.main()
